package org.ifmo.refrigerator.components;

import org.ifmo.refrigerator.debug.AutomatLog;


/**
 *  " "
 */
public class AirFilter extends BaseAutomatComponent {
    private final static int POLLUTION_EFFICIENCY = -2;
    private final static int TIME_TO_LIFE = 1000;

/**
 *  
 *     - Content
 */
    public AirFilter(Content content) {
        super(content);
    }
/**
 *  "    "
 */
    public void badAir() {
        A0(E1_BAD_AIR);
    }
/**
 *  "    "
 */
    public void goodAir() {
        A0(E2_GOOD_AIR);
    }
/**
 *  "    "
 */
    public void breakFilter() {
        timeToLife = 0;
        A0(E3_BAD_FILTER);
    }
/**
 *  " "
 */
    public void fixFilter() {
        timeToLife = TIME_TO_LIFE;
        A0(E4_FIX_FILTER);
    }

    // 
    private static final int E1_BAD_AIR = 1;
    private static final int E2_GOOD_AIR = 2;
    private static final int E3_BAD_FILTER = 3;
    private static final int E4_FIX_FILTER = 4;

    // 
    private static final int Y0_CONDITIONER_OFF = 0;
    private static final int Y1_CONDITIONER_ON = 1;
    private static final int Y2_CONDITIONER_BROKEN = 2;

    //  
    private int Y = Y0_CONDITIONER_OFF;

/**
 *  0
 */
    private void A0(int e) {
        int oldY = Y;
        switch (Y) {
            case Y0_CONDITIONER_OFF:
                if (e == E1_BAD_AIR) {
                    Y = Y1_CONDITIONER_ON;
                    z1();
                } else
                if (e == E3_BAD_FILTER) {
                    Y = Y2_CONDITIONER_BROKEN;
                    z2();
                    z3();
                }
                break;

            case Y1_CONDITIONER_ON:
                if (e == E2_GOOD_AIR) {
                    Y = Y0_CONDITIONER_OFF;
                    z2();
                } else
                if (e == E3_BAD_FILTER) {
                    Y = Y2_CONDITIONER_BROKEN;
                    z2();
                    z3();
                }
                break;

            case Y2_CONDITIONER_BROKEN:
                if (e == E4_FIX_FILTER) {
                    Y = Y1_CONDITIONER_ON;
                    z1();
                    z4();
                }
                break;
        }
        if (Y != oldY)
            AutomatLog.transaction(0, getAutomatName(), oldY, Y);
    }

    //   "     "
    private void z1() {
        isRunning = true;
        getContent().getAirFilterIndicator().On();
    }
    //   "     "
    private void z2() {
        isRunning = false;
        getContent().getAirFilterIndicator().Off();
    }
    //   "  " "
    private void z3() {
        getContent().getAirFilterBrokenIndicator().On();
    }
    //   "  " "
    private void z4() {
        getContent().getAirFilterBrokenIndicator().Off();
    }

    private boolean isRunning = false;
    private int timeToLife = TIME_TO_LIFE;

    //     
    public void timeEvent() {
        if ((timeToLife > 0) && isRunning) {
            getContent().getAir().changePollution(POLLUTION_EFFICIENCY);
            timeToLife--;
            if (timeToLife == 0) {
                breakFilter();
            }
        }
    }

    //    
    public int getTimeToLife() {
        return timeToLife;
    }

    public int getMaxTimeToLife() {
        return TIME_TO_LIFE;
    }
}
