package org.ifmo.refrigerator.components;

import org.ifmo.refrigerator.debug.AutomatLog;


/**
 *   
 */
public class Door extends BaseAutomatComponent {
    private final static int DELAY_15 = 50;
    private final static int DELAY_60 = 150;
    private int openDelay = 0;

    private Indicator indicator = null;
    private Lamp lamp = null;

/**
 * 
 */
    public Door(Content content, Indicator indicator, Lamp lamp) {
        super(content);
        this.indicator = indicator;
        this.lamp = lamp;
        indicator.Off();
        lamp.Off();
    }

    //   
    public boolean isDoorOpen() {
        return (Y != Y0_CLOSED);
    }

    //  " "
    public void doorOpened() {
        A1(E1_DOOR_OPENED);
    }
    //  " "
    public void doorClosed() {
        A1(E2_DOOR_CLOSED);
    }

    //  "  > 15."
    public void timeOut15() {
        A1(E3_OPENED_15);
    }

    //  "  > 60."
    public void timeOut60() {
        A1(E4_OPENED_60);
    }

    // 
    private static final int E1_DOOR_OPENED = 1;
    private static final int E2_DOOR_CLOSED = 2;
    private static final int E3_OPENED_15 = 3;
    private static final int E4_OPENED_60 = 4;

    // 
    private static final int Y0_CLOSED = 0;
    private static final int Y1_OPENED = 1;
    private static final int Y2_OPENED_15 = 2;
    private static final int Y3_OPENED_60 = 3;

    //  
    private int Y = Y0_CLOSED;

/**
 *  1
 */
    private void A1(int e) {
        int oldY = Y;
        switch (Y) {
            case Y0_CLOSED:
                if (e == E1_DOOR_OPENED) {
                    Y = Y1_OPENED;
                    z1();
                    z7();
                }
                break;

            case Y1_OPENED:
                if (e == E2_DOOR_CLOSED) {
                    Y = Y0_CLOSED;
                    z2();
                    z8();
                } else if (e == E3_OPENED_15) {
                    Y = Y2_OPENED_15;
                    z3();
                }
                break;

            case Y2_OPENED_15:
                if (e == E2_DOOR_CLOSED) {
                    Y = Y0_CLOSED;
                    z2();
                    z4();
                    z8();
                } else if (e == E4_OPENED_60) {
                    Y = Y3_OPENED_60;
                    z4();
                    z5();
                }
                break;

            case Y3_OPENED_60:
                if (e == E2_DOOR_CLOSED) {
                    Y = Y0_CLOSED;
                    z2();
                    z6();
                    z8();
                }
                break;
        }
        if (Y != oldY)
            AutomatLog.transaction(1, getAutomatName(), oldY, Y);
    }

    //   "   "
    private void z1() {
        indicator.On();
    }

    //   "   "
    private void z2() {
        indicator.Off();
    }

    //   "  "
    private void z3() {
        getContent().getLoudSpeaker().lowSoundOn();
    }

    //   "  "
    private void z4() {
        getContent().getLoudSpeaker().lowSoundOff();
    }

    //   "   "
    private void z5() {
        getContent().getLoudSpeaker().loudSoundOn();
    }

    //   "   "
    private void z6() {
        getContent().getLoudSpeaker().loudSoundOff();
    }

    //   "  "
    private void z7() {
        lamp.On();
    }

    //   "  "
    private void z8() {
        lamp.Off();
    }

    //   
    public void timeEvent() {
        if (isDoorOpen()) {
            openDelay++;
            if (openDelay == DELAY_15)
                timeOut15();
            else if (openDelay == DELAY_60)
                timeOut60();
        } else {
            openDelay = 0;
        }
    }
}

