package org.ifmo.refrigerator.ui;

import org.ifmo.refrigerator.components.Content;

import java.util.Timer;
import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.util.TimerTask;

/**
 * ,      .
 */
public class DisplayPanel extends JPanel {
    private static final int DEFAULT_WIDTH = 600;
    private static final int DEFAULT_HEIGHT = 600;
    private static final int DETAILS_X = 140;
    private static final int DETAILS_Y = 40;

    public static final int TYPE_TEMPERATURE = 0;
    public static final int TYPE_ELECTRICITY = 1;
    public static final int TYPE_CONDITIONER = 2;
    public static final int TYPE_CONDITIONER_ACTIVE_STATE = 3;
    public static final int TYPE_CONDITIONER_FILTER_ACTIVE_STATE = 4;
    public static final int TYPE_FREEZER_ICE_ACTIVE_STATE = 5;
    public static final int TYPE_FREEZER_DOOR_ACTIVESTATE = 6;
    public static final int TYPE_REFR_DOOR_ACTIVE_STATE = 7;
    public static final int DOOR_X_COORD[] = new int[] {490, 515, 563, 594};
    public static final int DOOR_Y_COORD[] = new int[] {65, 65, 65, 63};
    public static final int BDOOR_X_COORD[] = new int[] {490, 515, 563, 594};
    public static final int BDOOR_Y_COORD[] = new int[] {225, 225, 225, 223};

    private ImageLoader imageLoader = null;
    private Content content = null;

    private boolean isTopDoorOpening = true;
    private int nTopDoorFrame = 0;
    private boolean isBottomDoorOpening = true;
    private int nBottomDoorFrame = 0;

    private boolean breakFilterButtonOn = false;
    private boolean fixFilterButtonOn = false;
    private boolean removeIceButtonOn = false;
    private boolean pressedButtonOn = false;

    private final static int SENSOR_NONE = 0;
    private final static int SENSOR_FRAZIL = 1;
    private final static int SENSOR_AIR_FILTER = 2;
    private int sensorSelected = SENSOR_NONE;

/**
 *   .
 */
    private class CursorController extends MouseMotionAdapter {
/**
 *        
 */
            public void mouseMoved(MouseEvent e) {
                if ((e.getX() > 650 && e.getY() < 90)  ||
                    (e.getX() < 115 && e.getY() < 155) ||
                    (e.getX() > 490 && e.getX() < 610 && e.getY() > 60 && e.getY() < 340)) {
                    setCursor(new Cursor(Cursor.HAND_CURSOR));
                } else {
                    setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                }
            }
/**
 *      
 */
            public void mouseDragged(MouseEvent e) {
                pressedButtonOn = false;
                if (e.getX() > 650 && e.getY() < 30)
                    pressedButtonOn = breakFilterButtonOn;
                else if (e.getX() > 650 && e.getY() >= 30 && e.getY() < 60)
                    pressedButtonOn = fixFilterButtonOn;
                else if (e.getX() > 650 && e.getY() >= 60 && e.getY() < 90)
                    pressedButtonOn = removeIceButtonOn;
                repaint();
            }
        }

/**
 *   
 */
    private class MouseListener extends MouseAdapter {

    /**
     *     
     */
        private void testSensorButtons(Point mouse) {
            final Rectangle rectFrazil = new Rectangle(0, 0, 112, 75);
            final Rectangle rectAirFilter = new Rectangle(0, 76, 112, 75);
            final Rectangle rectNone = new Rectangle(0, 150, 112, 450);
            if (rectFrazil.contains(mouse))
                sensorSelected = SENSOR_FRAZIL;
            else if (rectAirFilter.contains(mouse))
                sensorSelected = SENSOR_AIR_FILTER;
            else if (rectNone.contains(mouse))
                sensorSelected = SENSOR_NONE;
        }

/**
 *      
 */
        private void testDoors(Point mouse) {
            final Rectangle topDoor = new Rectangle(490, 60, 120, 165);
            final Rectangle bottomDoor = new Rectangle(490, 225, 120, 115);
            if (topDoor.contains(mouse)) {
                isTopDoorOpening = !content.getTopDoor().isDoorOpen();
                if (isTopDoorOpening)
                    content.getTopDoor().doorOpened();
                else
                    content.getTopDoor().doorClosed();

                nTopDoorFrame = 0;
                Timer timer = new Timer();
                timer.schedule(new TimerTask() {
                    public void run() {
                        while (nTopDoorFrame < (imageLoader.REFR_DOOR_IMAGES_CNT - 1)) {
                            repaint();
                            nTopDoorFrame++;
                            try {
                                Thread.sleep(100);
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                        repaint();
                    }
                }, 0);
            } else if (bottomDoor.contains(mouse)) {
                isBottomDoorOpening = !content.getBottomDoor().isDoorOpen();
                if (isBottomDoorOpening)
                    content.getBottomDoor().doorOpened();
                else
                    content.getBottomDoor().doorClosed();
                nBottomDoorFrame = 0;
                Timer timer = new Timer();
                timer.schedule(new TimerTask() {
                    public void run() {
                        while (nBottomDoorFrame < (imageLoader.MOROZ_DOOR_IMAGES_CNT - 1)) {
                            repaint();
                            nBottomDoorFrame++;
                            try {
                                Thread.sleep(100);
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                        repaint();
                    }
                }, 0);
            }
        }

/**
 *  
 */
        public void mousePressed(MouseEvent e) {
            testDoors(e.getPoint());
            testSensorButtons(e.getPoint());
            pressedButtonOn = true;
            if (e.getX() > 650 && e.getY() < 30)
                breakFilterButtonOn = true;
            else if (e.getX() > 650 && e.getY() >= 30 && e.getY() < 60)
                fixFilterButtonOn = true;
            else if (e.getX() > 650 && e.getY() >= 60 && e.getY() < 90)
                removeIceButtonOn = true;
            else
                pressedButtonOn = false;
            repaint();
        }

/**
 *  
 */
        public void mouseReleased(MouseEvent e) {
            if (pressedButtonOn) {
                if (breakFilterButtonOn)
                    content.getAirFilter().breakFilter();
                else if (fixFilterButtonOn)
                    content.getAirFilter().fixFilter();
                else if (removeIceButtonOn)
                    content.getFrazilController().cleanFrazil();
            }
            breakFilterButtonOn = false;
            fixFilterButtonOn = false;
            removeIceButtonOn = false;
            repaint();
        }
    }

/**
 *    DisplayPanel
 */
    public DisplayPanel(ImageLoader imgLoader, Content content) {
        this.imageLoader = imgLoader;
        this.content = content;
        this.isTopDoorOpening = !content.getTopDoor().isDoorOpen();
        this.isBottomDoorOpening = !content.getBottomDoor().isDoorOpen();
        this.setPreferredSize(new Dimension(DEFAULT_WIDTH, DEFAULT_HEIGHT));
        this.addMouseMotionListener(new CursorController());
        this.addMouseListener(new MouseListener());
    }

/**
 *  
 */
    private void drawScale(Graphics g, String text, int x, int y, int value, int maxValue, Color lineColor) {
        final int scaleWidth = imageLoader.imgScale.getWidth(null);
        g.drawString(text, x, y);
        y += g.getFontMetrics().getHeight();
        g.drawImage(imageLoader.imgScale, x, y, null);
        Color tColor = g.getColor();
        g.setColor(lineColor);
        g.fillRect(x, y, (scaleWidth * value) / maxValue, 2);
        g.setColor(tColor);
        g.drawString(Integer.toString(value), x + scaleWidth, y);
    }

/**
 * 
 */
    public void paint(Graphics g) {
        //--------- ---------------
        g.drawImage(imageLoader.imgDisplay, 0, 0, null);

        //----------------------
        g.drawImage(imageLoader.imgRefrOpen, 490, 60, null);
        if (isTopDoorOpening)
            g.drawImage(imageLoader.imgRefrDoor[nTopDoorFrame], DOOR_X_COORD[nTopDoorFrame], DOOR_Y_COORD[nTopDoorFrame], null);
        else
            g.drawImage(imageLoader.imgRefrDoor[3 - nTopDoorFrame], DOOR_X_COORD[3 - nTopDoorFrame], DOOR_Y_COORD[3 - nTopDoorFrame], null);
        if (isBottomDoorOpening)
            g.drawImage(imageLoader.imgMorozDoor[nBottomDoorFrame], BDOOR_X_COORD[nBottomDoorFrame], BDOOR_Y_COORD[nBottomDoorFrame], null);
        else
            g.drawImage(imageLoader.imgMorozDoor[3 - nBottomDoorFrame], BDOOR_X_COORD[3 - nBottomDoorFrame], BDOOR_Y_COORD[3 - nBottomDoorFrame], null);

        //-----------------------------
        if (pressedButtonOn) {
            if (breakFilterButtonOn) g.drawImage(imageLoader.imgBreakFilterButtonOn, 650, 0, null);
            if (fixFilterButtonOn) g.drawImage(imageLoader.imgFixFilterButtonOn, 650, 30, null);
            if (removeIceButtonOn) g.drawImage(imageLoader.imgRemoveIceButtonOn, 650, 60, null);
        }

        //--------------------------
        if (content.getTopIndicator().isLighting())
            g.drawImage(imageLoader.imgRefrDoorActiveState, 52, 420, null);
        if (content.getBottomIndicator().isLighting())
            g.drawImage(imageLoader.imgFreezerDoorActiveState, 30, 420, null);
        if (content.getAirFilterBrokenIndicator().isLighting())
            g.drawImage(imageLoader.imgConditionerFilterActiveState, 8, 398, null);
        if (content.getAirFilterIndicator().isLighting())
            g.drawImage(imageLoader.imgConditionerActiveState, 8, 420, null);
        if (content.getFrazilIndicator().isLighting())
            g.drawImage(imageLoader.imgFreezerIceActiveState, 30, 398, null);
        if (content.getTopLamp().isOn())
            g.drawImage(imageLoader.imgLightOn, 460, 80, null);
        else g.drawImage(imageLoader.imgLightOff, 460, 80, null);
        if (!content.getBottomLamp().isOn()) {
            g.drawImage(imageLoader.imgLightOff, 460, 245, null);
        } else {
            g.drawImage(imageLoader.imgLightOn, 460, 245, null);
        }

        //----------------------
        switch (sensorSelected) {
            case SENSOR_FRAZIL:
                g.drawImage(imageLoader.imgTemperatureActive, 0, 0, null);
                drawScale(g, "Frazil", DETAILS_X, DETAILS_Y,
                        content.getFrazil().getFrazil(),
                        content.getFrazil().MAX_FRAZIL,
                        Color.blue);
                break;
            case SENSOR_AIR_FILTER:
                g.drawImage(imageLoader.imgConditionerActive, 0, 75, null);
                drawScale(g, "Air", DETAILS_X, DETAILS_Y,
                        content.getAir().getPollution(),
                        content.getAir().MAX_POLLUTION,
                        Color.red);
                drawScale(g, "Filter", DETAILS_X, DETAILS_Y + 200,
                        content.getAirFilter().getMaxTimeToLife() - content.getAirFilter().getTimeToLife(),
                        content.getAirFilter().getMaxTimeToLife(),
                        Color.red);
                break;
            case SENSOR_NONE:
                break;
        }
    }
}
